package com.link.core.converter;

import com.link.core.converter.util.MapEntry;
import com.link.core.converter.util.RegularCloneable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class ArrayListMapEntryConverter<SK, SV, TK, TV, E extends MapEntry<TK, TV>> implements Converter<Map<SK, SV>, ArrayList<E>>, HasConverters, RegularCloneable {
   private static final boolean DEFAULT_OPTIMAL = true;
   private Class<E> elementType;
   private Converter<SK, TK> keyConverter;
   private Converter<SV, TV> valueConverter;
   private boolean optimal = true;

   public static <K, V, E extends MapEntry<K, V>> ArrayListMapEntryConverter<K, V, K, V, E> newConverter(Class<K> keyType, Class<V> valueType, Class<E> elementType) {
      ArrayListMapEntryConverter<K, V, K, V, E> converter = new ArrayListMapEntryConverter();
      converter.setElementType(elementType);
      return converter;
   }

   public static <SK, TK, V, E extends MapEntry<TK, V>> ArrayListMapEntryConverter<SK, V, TK, V, E> newConverter(Converter<SK, TK> keyConverter, Class<V> valueType, Class<E> elementType) {
      ArrayListMapEntryConverter<SK, V, TK, V, E> converter = new ArrayListMapEntryConverter();
      converter.setElementType(elementType);
      converter.setKeyConverter(keyConverter);
      return converter;
   }

   public static <K, SV, TV, E extends MapEntry<K, TV>> ArrayListMapEntryConverter<K, SV, K, TV, E> newConverter(Class<K> keyType, Converter<SV, TV> valueConverter, Class<E> elementType) {
      ArrayListMapEntryConverter<K, SV, K, TV, E> converter = new ArrayListMapEntryConverter();
      converter.setElementType(elementType);
      converter.setValueConverter(valueConverter);
      return converter;
   }

   public static <SK, SV, TK, TV, E extends MapEntry<TK, TV>> ArrayListMapEntryConverter<SK, SV, TK, TV, E> newConverter(Class<SK> sourceKeyType, Class<SV> sourceValueType, Class<TK> targetKeyType, Class<TV> targetValueType, Class<E> elementType) {
      ArrayListMapEntryConverter<SK, SV, TK, TV, E> converter = new ArrayListMapEntryConverter();
      converter.setElementType(elementType);
      return converter;
   }

   public static <SK, SV, TK, TV, E extends MapEntry<TK, TV>> ArrayListMapEntryConverter<SK, SV, TK, TV, E> newConverter(Converter<SK, TK> keyConverter, Converter<SV, TV> valueConverter, Class<E> elementType) {
      ArrayListMapEntryConverter<SK, SV, TK, TV, E> converter = new ArrayListMapEntryConverter();
      converter.setElementType(elementType);
      converter.setKeyConverter(keyConverter);
      converter.setValueConverter(valueConverter);
      return converter;
   }

   protected ArrayListMapEntryConverter() {
   }

   public Class<E> getElementType() {
      return this.elementType;
   }

   public ArrayListMapEntryConverter<SK, SV, TK, TV, E> setElementType(Class<E> elementType) {
      this.elementType = elementType;
      return this;
   }

   public Converter<SK, TK> getKeyConverter() {
      return this.keyConverter;
   }

   public ArrayListMapEntryConverter<SK, SV, TK, TV, E> setKeyConverter(Converter<SK, TK> keyConverter) {
      this.keyConverter = keyConverter;
      return this;
   }

   public Converter<SV, TV> getValueConverter() {
      return this.valueConverter;
   }

   public ArrayListMapEntryConverter<SK, SV, TK, TV, E> setValueConverter(Converter<SV, TV> valueConverter) {
      this.valueConverter = valueConverter;
      return this;
   }

   public boolean isOptimal() {
      return this.optimal;
   }

   public ArrayListMapEntryConverter<SK, SV, TK, TV, E> setOptimal(boolean optimal) {
      this.optimal = optimal;
      return this;
   }

   public List<Converter> getConverters() {
      List<Converter> list = new ArrayList();
      if (this.keyConverter != null) {
         list.add(this.keyConverter);
      }

      if (this.valueConverter != null) {
         list.add(this.valueConverter);
      }

      return list;
   }

   public ArrayList<E> convert(Map<SK, SV> source) throws ConversionException {
      if (source == null) {
         return null;
      } else {
         try {
            ArrayList<E> target = new ArrayList();
            Map<SK, TK> keyBuffer = new HashMap();
            Map<SV, TV> valueBuffer = new HashMap();
            Iterator var5 = source.entrySet().iterator();

            while(var5.hasNext()) {
               Map.Entry<SK, SV> sourceEntry = (Map.Entry)var5.next();
               E targetElement = this.newTargetElement();
               targetElement.setKey(this.convertKey(sourceEntry.getKey(), keyBuffer));
               targetElement.setValue(this.convertValue(sourceEntry.getValue(), valueBuffer));
               target.add(targetElement);
            }

            return target;
         } catch (ConversionException var8) {
            ConversionException e = var8;
            throw e;
         } catch (Exception var9) {
            Exception e = var9;
            throw new ConversionException(e);
         }
      }
   }

   public Object clone() {
      ArrayListMapEntryConverter<SK, SV, TK, TV, E> other = new ArrayListMapEntryConverter();
      other.copyFrom(this);
      return other;
   }

   public void copyFrom(Object source) {
      if (source != null && source instanceof ArrayListMapEntryConverter) {
         ArrayListMapEntryConverter<SK, SV, TK, TV, E> sc = (ArrayListMapEntryConverter)source;
         this.elementType = sc.elementType;
         if (sc.keyConverter instanceof RegularCloneable) {
            this.keyConverter = (Converter)((RegularCloneable)sc.keyConverter).clone();
         } else {
            this.keyConverter = sc.keyConverter;
         }

         if (sc.valueConverter instanceof RegularCloneable) {
            this.valueConverter = (Converter)((RegularCloneable)sc.valueConverter).clone();
         } else {
            this.valueConverter = sc.valueConverter;
         }

         this.optimal = sc.optimal;
      }
   }

   private E newTargetElement() throws Exception {
      return this.elementType == null ? new MapEntry() : (MapEntry)this.elementType.newInstance();
   }

   private TK convertKey(SK source, Map<SK, TK> buffer) throws ConversionException {
      if (this.keyConverter == null) {
         return source;
      } else if (!this.optimal) {
         return this.keyConverter.convert(source);
      } else {
         assert buffer != null;

         if (buffer.containsKey(source)) {
            return buffer.get(source);
         } else {
            TK target = this.keyConverter.convert(source);
            buffer.put(source, target);
            return target;
         }
      }
   }

   private TV convertValue(SV source, Map<SV, TV> buffer) throws ConversionException {
      if (this.valueConverter == null) {
         return source;
      } else if (!this.optimal) {
         return this.valueConverter.convert(source);
      } else {
         assert buffer != null;

         if (buffer.containsKey(source)) {
            return buffer.get(source);
         } else {
            TV target = this.valueConverter.convert(source);
            buffer.put(source, target);
            return target;
         }
      }
   }
}
